/************ mime.c
	MIME and quoted-printable handling routines for nlist
		Written by Gabor Paller
				*************/

#include	<stdio.h>
#include	<string.h>
#define		N_EXTERN extern
#include	"config.h"
#include	"commonvars.h"
				
/* Processes the letter subject regarding the list title.
   Removes all occurences of the title string from the subject
   then appends it to the front of the subject and returns the
   new subject */
char *process_list_title( char *subject, char *title ) {
  char result[N_BUFSIZE];
  static char result2[N_BUFSIZE];
  static char result3[3*N_BUFSIZE];
  char	*p,*p1,*ep,*target;
  int	flen,plen;
  int	charset;
  bool	refound = FALSE;
  
/* Decode the subject string from quoted format */
  charset = decode_quoted_fragments( subject );
/* Processes the subject */
  p1 = subject;
  plen = strlen( title );
  target = result;
  do {
    p = strstr( p1,title );
    ep = p == NULL ? subject + strlen( subject ) : p;
    flen = ep - p1;
/* Remove extra spaces after Re:, etc. */
    if( ( flen > 0 ) && ( *(ep-1) == ' ' ) ) {
      --flen;
      --ep;
    } 
    if( flen != 0 )
      strncpy( target,p1,flen );
    target += flen;
    *target = 0;   
    if( p != NULL )
      p1 = p + plen;
  } while( p != NULL );
/* Now collapse multiple Re:-s into one */
  target = p = result;
  refound = FALSE;
  while( *p ) {
    if( ( strncasecmp( p,"Re: ",4 ) == 0 ) ||
        ( strncasecmp( p,"aw: ",4 ) == 0 ) ||
        ( strncasecmp( p,"v: ",4 ) == 0 ) ||
        ( strncasecmp( p,"V: ",4 ) == 0 ) ) {
      if( !refound ) {
        strncpy( target,"Re: ",4 );
        target += 4;
        refound = TRUE;
      }
      p += 4;
    }
    else {
      *(target++) = *(p++);
      refound = FALSE;
    }  
  }
  *target = 0;
  strcpy( result2,title );
  if( ( *result != ' ' ) && ( *result != 0 ) )
  	strcat( result2," " );
  strcat( result2,result );
  encode_quoted_printable( result2,result3,charset );
  return result3;
}

void write_banner_file( FILE *nf, FILE *banner ) {
  char buf[N_BUFSIZE+1];

  while( fgets(buf, N_BUFSIZE, banner ) != NULL )
    fputs( buf,nf );
}

/* Inserts the sender banner into the letter file. The function processes
   MIME parts, if the letter is composed of MIME parts then the sender banner will
   be inserted into the first MIME part having text/plain content type */
void insert_senderbanner( char tempfilename[] ) {
  char	tempfilename2[N_BUFSIZE];
  char	line[N_BUFSIZE];
  FILE	*nf,*of,*banner;
  
#define DONT_SEARCH_CONTENT	0	/* text/plain content has been found, don't search for it anymore */
#define SEARCH_PLAIN_CONTENT	1	/* Search the content after the message headers */
#define SEARCH_MIME_CONTENT	2	/* Search for content after the MIME headers */
#define SEARCH_MIME_TYPE	3	/* Search MIME type header */
#define SEARCH_MIME_END		4	/* Search end of the MIME part */
  int	SearchContentState;
  bool	MessageHeaderEnd = FALSE;  

  sprintf( tempfilename2,"%s/%s.banner",PATH,list_name );
  banner = fopen( tempfilename2,"r" );
/* Return if banner file doesn't exist */
  if( banner == NULL )
  	return;  
  strcpy( tempfilename2,tempfilename );
  strcat( tempfilename2,"2" );
/* Renames the message file so that it can be recreated under the old name */
  rename( tempfilename,tempfilename2 );
  if( ( nf = file_open( tempfilename,"w" ) ) == NULL )
  	ehandle( 1,"Cannot create new message file: %s",tempfilename );
  of = file_open( tempfilename2,"r" );
  SearchContentState = *boundary == 0 ?
			SEARCH_PLAIN_CONTENT : SEARCH_MIME_CONTENT;
  while( fgets( line,N_BUFSIZE-1,of ) != NULL ) {
    if( !MessageHeaderEnd ) {
      fputs( line,nf );
      if( *line == '\n' ) {
        MessageHeaderEnd = TRUE;
        ContentPointer = ftell( nf );
      }
      else
	      continue;
    }
    if( SearchContentState != SEARCH_MIME_END )
		fputs( line,nf );
    if( SearchContentState == SEARCH_MIME_END ) {
      if( strstr( line,boundary ) != NULL ) {
        SearchContentState = DONT_SEARCH_CONTENT;
        write_banner_file( nf,banner );
/* Let's leave one empty line between the end of the banner and the
   MIME boundary. Not really necessary, just to be sure .., */
        fputs( "\n",nf );
      }
      fputs( line, nf );
    } else 
    if( ( *line == '\n' ) && ( SearchContentState != DONT_SEARCH_CONTENT ) ) {
/* If this is the end of the message headers, (not MIME part headers), we
   register the content offset. Content offset may change from the original
   message due to e.g. CR-LF translation */
      if( !MessageHeaderEnd ) {
        ContentPointer = ftell( nf );
        MessageHeaderEnd = TRUE;
      }
      if( SearchContentState == SEARCH_PLAIN_CONTENT ) {
        SearchContentState = *boundary == 0 ? DONT_SEARCH_CONTENT : SEARCH_MIME_END;
        decode_quoted_fragments( sender_full_address );
        sprintf( line,INTL_SENDER,sender_full_address );
        fputs( line,nf );
      }
    } else
    if( ( SearchContentState == SEARCH_MIME_CONTENT ) &&
        ( strstr( line,boundary ) != NULL ) ) {
      SearchContentState = SEARCH_MIME_TYPE;
    } else
/* Check to see if we have the text/plain content part in MIME content */
    if( ( SearchContentState == SEARCH_MIME_TYPE ) &&
    	( strncasecmp( line,"Content-Type:",13 ) == 0 ) &&
    	( strcasestr( line,"text/plain" ) != NULL ) ) {
      SearchContentState = SEARCH_PLAIN_CONTENT;
    }
  }
  if( *boundary == 0 )
    write_banner_file( nf, banner );
  fclose( nf );
  fclose( of );
  fclose( banner );
  unlink( tempfilename2 );
}

/* Compiles a (probably multi-line) mail header into one string. Gets pointer to
   the header part of the first line, destination string, maximal length of the
   destination string and message file descriptor. Returns pointer to the first
   line that doesn't belong to this header line */
char *CompileMultiLineHeader( char *firstline, char dest[], int maxlen, FILE *mf ) {
  char	*nl,*p;

  strcpy( dest, firstline );
  if( mf != NULL )
	fprintf( mf,"%s\n",firstline );
  do {
    nl = getline( stdin );
    if( ( nl == NULL ) || ( *nl == 0 ) || ( *nl == '\n' ) )
		break;
/* Break the loop if the line doesn't start with a white space (tab or space),
   in this case we have reached the end of the multi-line header */
    if( ( *nl != '\t' ) && ( *nl != ' ' ) )
    		break;
    if( mf != NULL )
	fprintf( mf,"%s\n",nl );
    p = nl;
/* Skip white spaces at the beginning of header */
    while( ( *p == ' ' ) || ( *p == '\t' ) )
    	++p;
    if( ( strlen( dest ) + strlen( p ) ) < maxlen )
	strcat( dest,p ); 		 
  } while( TRUE );
  return nl;
}

/* Scans the letter if there are attachment of forbidden extensions.
   Note that the file extension is checked instead of MIME type because
   extension is used to launch attachments on Windows systems.
   Returns true if the message contains attachments of forbidden extensions */
bool CheckForbiddenExtensions( char TempFileName[] ) {
#define MAX_EXT	256
  FILE	*tempfile;
  char	parsed_ext_list[N_BUFSIZE];
  char	*ext_list[MAX_EXT];
  int	i,state;
  char	*p,*p2,*s;
  char	buffer[N_BUFSIZE];
  char	filename[N_BUFSIZE];
  
  if( *boundary == 0 )
  	return FALSE;	/* Return if no MIME parts */
  strcpy( parsed_ext_list,filter_extensions );
  i = 0;
  s = parsed_ext_list;
  do {
    p = strtok( s,", \t" );
    s = NULL;
    if( ( ext_list[i] = p ) == NULL )
    	break;
    if( ++i >= MAX_EXT )
    	break;
  } while( p != NULL );
  if( ( tempfile = fopen( TempFileName,"r" ) ) == NULL )
      return FALSE;
  fseek( tempfile,ContentPointer,SEEK_SET );
  state = SEARCH_MIME_CONTENT;
  while( fgets( buffer,N_BUFSIZE-1,tempfile ) != NULL ) {
    if( ( state == SEARCH_MIME_CONTENT ) &&
        ( strstr( buffer,boundary ) != NULL ) ) {
      state = SEARCH_MIME_TYPE;
    } else
    if( state == SEARCH_MIME_TYPE ) {
      if( *buffer == '\n' ) {
        state = SEARCH_MIME_CONTENT;
        continue;
      }
      p = strstr( buffer,"name=" );
      if( p != NULL )
      	p += 5;
      else {
        p = strstr( buffer,"ord=" );
        if( p != NULL )
          p += 4;
        else {
          p = strstr( buffer,"filename=" );
          if( p != NULL )
        	p += 9;
        }
      }
      if( p == NULL )
      	continue;
/* White space is not allowed here as separator character because
   some viruses omit the quotes even if the name contains spaces
   and Outlook accepts it as if there were quotes. */
      getWord( p,filename,N_BUFSIZE,";" );
      if( ( p = strrchr( filename,'.' ) ) == NULL )
      	continue;
      ++p;
      i = 0;
      while( ext_list[i] != NULL ) {
        if( strcasecmp( p,ext_list[i] ) == 0 ) {	/* Barred extension found */
          Log( 2,1,"Attached file %s from %s has barred extension %s",
          		filename,addr,ext_list[i] );
          fseek( tempfile,0L,SEEK_SET );
          while( fgets( buffer,N_BUFSIZE-1,tempfile ) != NULL ) {
	    if( ( p2 = strchr( buffer,'\n' ) ) != NULL )
	          *p2 = 0;
            Log( 4,0,"EXTFILT: %s",buffer );
          }
          fclose( tempfile );
          return TRUE;
        }
        ++i;
      }
      continue;		/* No problem with the extension */
    } 
  }
  fclose( tempfile );
  return FALSE;
}
